/*
 * Copyright 2017 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.jbaesner.playground.jpa;

import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Singleton
@Startup
public class TimeTestStartupBean {

	private static final Logger logger = LoggerFactory.getLogger(TimeTestStartupBean.class);

	ZoneId zoneId = ZoneId.of("Europe/Berlin");

	@PersistenceContext
	private EntityManager em;

	@PostConstruct
	public void customerStart() {

		logger.info("user.timezone '{}'", System.getProperty("user.timezone"));

		ZonedDateTime start = LocalDateTime.of(2017, 10, 29, 1, 0, 0).atZone(zoneId);
		ZonedDateTime end = start.plusHours(3);

		int i = 0;
		while (start.isBefore(end)) {

			print(start, i);
			TimeTestEntityIdOnTs tte = new TimeTestEntityIdOnTs();
			tte.setTs(start);
			tte.setValue(i);
			em.persist(tte);
			em.flush();

			TimeTestEntityIdOnValue tteval = new TimeTestEntityIdOnValue();
			tteval.setTs(start);
			tteval.setValue(i);
			em.persist(tteval);
			em.flush();

			start = start.plusHours(1);
			i++;
		}

		logger.info("------------------------------------------------------------------------------------------------");
		logger.info("-- Now read with Id on Value --");
		logger.info("-- here all ZonedDateTimes are correct...");
		logger.info("------------------------------------------------------------------------------------------------");

		Query query = em.createQuery("SELECT e FROM TimeTestEntityIdOnValue e");

		@SuppressWarnings("unchecked")
		List<TimeTestEntityIdOnValue> ttesVal = query.getResultList();

		i = 0;
		for (TimeTestEntityIdOnValue tte : ttesVal) {
			ZonedDateTime zdt = tte.getTs();
			print(zdt, i);
			i++;
		}
		logger.info("------------------------------------------------------------------------------------------------");
		logger.info("-- Now read with Id on Timestamp --");
		logger.info("-- here the epoch[1] is wrong...");
		logger.info("------------------------------------------------------------------------------------------------");

		query = em.createQuery("SELECT e FROM TimeTestEntityIdOnTs e");

		@SuppressWarnings("unchecked")
		List<TimeTestEntityIdOnTs> ttesTs = query.getResultList();

		i = 0;
		for (TimeTestEntityIdOnTs tte : ttesTs) {
			ZonedDateTime zdt = tte.getTs();
			print(zdt, i);
			i++;
		}
		logger.info("------------------------------------------------------------------------------------------------");
	}

	private void print(ZonedDateTime zdt, int idx) {
		logger.info("epoch[" + idx + "]: " + zdt.toEpochSecond() + ", instant: " + zdt.toInstant() + ", toString: "
				+ zdt.toString() + ", withZoneSameInstant: " + zdt.withZoneSameInstant(zoneId) + ", offset: "
				+ zdt.getOffset());
	}

}
